require "ubus"
require "uloop"

json = require "json"

ubus_cellular = {}

local conn
local ubus_objects = {}
local timer
local data_status = "disconnected"


function ubus_cellular.init()
	uloop.init()
	uqmi.init()
	conn = ubus.connect()

	conn:add( ubus_objects())

	if not conn then
		error("Failed to connect to ubus")
	end
	timer = uloop.timer(t)
	timer:set(5000)
end

function ubus_cellular.connect()
	uloop.run()
end

function t()
	local status = conn:call("network.interface.IOT1", "status", {})
	if status["up"] then
		data_status = "connected"
	else
		data_status = "disconnected"
	end
	timer:set(5000)
end


function map_reg_to_reg_code(registration, roaming)
	local reg_code = 0
	local registration_str = "unknown"
	if  registration == "not_registered" 
	then
		reg_code = 0
	end
	if  registration == "registered" and roaming == false
	then
		reg_code = 1
		registration_str = "registered home"
	end
	if  registration == "registered" and roaming == true
	then
		reg_code = 5
		registration_str = "registered roaming"
	end
	if  registration == "searching" 
	then
		reg_code = 2
	end
	if  registration == "registering_denied" 
	then
		reg_code = 3
	end
	if  registration == "unknown" 
	then
		reg_code = 4
	end

	return reg_code, registration_str
end



function ubus_objects()
	ubus_objects={
		cellular = {
			status = {
			function(req, msg)
				local params = {}

				print ("Updating cellular status")
				for i, device in pairs(uqmi.get_devices()) do

					print ("Updating for device ".. device)
					local imei, rssi, net_type, registration, data_connectivity, roaming, plmn_mcc, plmn_mnc, registration_code, plmn_code

					imei = uqmi.get_imei(device)
					registration, plmn_mcc, plmn_mnc, roaming, plmn_description = uqmi.get_serving_system(device)
					registration_code, registration = map_reg_to_reg_code(registration, roaming)
					net_type, rssi = uqmi.get_signal_info(device)
					data_connectivity = uqmi.get_data_status(device)


					if plmn_mcc ~= nil and plmn_mnc ~= nil then
						plmn_code = plmn_mcc..plmn_mnc
					else
						plmn_code = ""
					end



					params[i] = {
						imei= imei:gsub("\"",""), 
						rssi = rssi, 
						net_type = net_type, 
						registration=registration, 
						registration_code=registration_code,
						data_connectivity = data_status, 
						roaming=tostring(roaming), 
						plmn_description = plmn_description,
						plmn_code = plmn_code,
						device = "LTE CAT-M/ NBIoT",
						port = device,
						sim_status = "NA"
						}
				end

				conn:reply(req, {message=json.encode(params)});
			end, {id = ubus.INT32, msg = ubus.STRING }
				}
			}
		}	
	return ubus_objects
end

return ubus_cellular
